#!/usr/bin/perl -w

# TODO: Write a smerge test that includes a copy from rev that isn't
# resolvable to the mirror's remote rev.  This resulted in mysterious
# 'file not found: revision 0' and is now fixed.

use Test::More tests => 1;

TODO: {
todo_skip "doesn't exist", 1;

}
